<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Реализует отгрузку товара в ТК.
 */
abstract class Shop_Order_Dispatcher
{
	/**
	 * Параметры посылки.
	 *
	 * @var array
	 */
	protected $_aParams = array(
		'width' => 0,
		'height' => 0,
		'depth' => 0
	);

	/**
	 * Таймаут запросов.
	 *
	 * @var integer
	 */
	protected $_apiTimeout = 30;

	/**
	 * Режим отладки.
	 *
	 * @var boolean
	 */
	protected $_debug = FALSE;

	/**
	 * Создает объект Shop_Order_Dispatcher.
	 *
	 * @param  string  $sProviderName
	 * @return Shop_Order_Dispatcher
	 */
	static public function factory($sProviderName)
	{
		$sProvider = strtolower($sProviderName);
		$sProviderClassName = __CLASS__ . '_' . ucfirst($sProviderName);

		if (!class_exists($sProviderClassName))
		{
			throw new Core_Exception("Class for provider \"{$sProviderName}\" not found.");
		}

		$aConfig = Core_Config::instance()->get('shop_order_dispatcher_config');

		if (!isset($aConfig[$sProvider]))
		{
			throw new Core_Exception("Config for provider \"{$sProviderName}\" not loaded.");
		}

		return new $sProviderClassName($aConfig[$sProvider]);
	}

	/**
	 * Конструктор.
	 *
	 * @param  array  $aConfig
	 * @return void
	 */
	abstract public function __construct(array $aConfig);

	/**
	 * Изменяет режим отладки.
	 *
	 * @param  boolean  $debug
	 * @return self
	 */
	public function debug($debug)
	{
		$this->_debug = (boolean) $debug;

		return $this;
	}

	/**
	 * Отправляет заказ в службу доставки.
	 *
	 * @param  Shop_Order_Model  $oShopOrder
	 * @return self
	 */
	public function dispatch(Shop_Order_Model $oShopOrder)
	{
		$this->_dispatch($oShopOrder);

		return $this;
	}

	/**
	 * Абстрактный метод для реализации стратегии.
	 *
	 * @param  Shop_Order_Model  $oShopOrder
	 * @return void
	 */
	abstract protected function _dispatch(Shop_Order_Model $oShopOrder);

	/**
	 * Логирует строку.
	 *
	 * @param  string  $log
	 * @return void
	 */
	protected function _log($log)
	{
		if ($this->_debug)
		{
			print $log;
		}
		else
		{
			Core_Log::instance()->clear()
				->status(Core_Log::$NOTICE)
				->write($log);
		}
	}

	/**
	 * Сообщает об ошибке.
	 *
	 * @param  string  $error
	 * @return void
	 */
	protected function _reportError($error)
	{
		if ($this->_debug)
		{
			throw new \Core_Exception($error);
		}
	}

	/**
	 * Возвращает значение доп. свойства.
	 *
	 * @param  Shop_Order_Model  $oShopOrder
	 * @param  string  $tagName
	 * @return mixed
	 */
	protected function _getPropertyValue(Shop_Order_Model $oShopOrder, $tagName)
	{
		$oShopOrderPropertyList = Core_Entity::factory('Shop_Order_Property_List', $oShopOrder->shop_id);
		$oProperty = $oShopOrderPropertyList->Properties->getByTagName($tagName);
		$aoPropertyValues = $oProperty->getValues($oShopOrder->id);

		return count($aoPropertyValues)
			? $aoPropertyValues[0]->value
			: NULL;
	}
}