<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Реализует отгрузку товара в ТК Hermes.
 */
class Shop_Order_Dispatcher_Hermes extends Shop_Order_Dispatcher
{
	/**
	 * URL для доступа к API.
	 *
	 * Тестовый: https://test-api.hermes-dpd.ru/WS/RestService.svc/rest/
	 * Продакшн: https://api.hermes-dpd.ru/WS/RestService.svc/rest/
	 *
	 * @var string
	 */
	protected $_apiUrl = 'https://api.hermes-dpd.ru/WS/RestService.svc/rest/';

	/**
	 * Логин для доступа к API.
	 *
	 * Для теста:
	 * testlogin
	 *
	 * @var string
	 */
	protected $_apiLogin;

	/**
	 * Пароль для доступа к API.
	 *
	 * Для теста:
	 * testpassword
	 *
	 * @var string
	 */
	protected $_apiPassword;

	/**
	 * Идентификатор бизнес-юнита клиента.
	 *
	 * @var integer
	 */
	protected $_bussinessUnitCode;

	/**
	 * {@inherit}
	 */
	public function __construct(array $aConfig)
	{
		$this->_apiUrl = Core_Array::get($aConfig, 'url');
		$this->_apiLogin = Core_Array::get($aConfig, 'login');
		$this->_apiPassword = Core_Array::get($aConfig, 'password');
		$this->_bussinessUnitCode = Core_Array::get($aConfig, 'bussinessUnitCode');
	}

	/**
	 * {@inherit}
	 */
	protected function _dispatch(Shop_Order_Model $oShopOrder)
	{
		$data = array(
			'preadvices' => array(
				array(
					'BusinessUnitCode' => $this->_bussinessUnitCode, // Идентификатор бизнес-юнита клиента
					'CashOnDeliveryCurrency' => 'RUB', // Валюта стоимости посылки
					'CashOnDeliveryValue' => 0, // Сумма
					'ClientOrderNumber' => $oShopOrder->invoice, // Номер заказа в системе клиента
					'ClientParcelNumber' => $oShopOrder->invoice, // Уникальный номер посылки в системе клиента
					'CustomerCity' => $oShopOrder->Shop_Country_Location_City->name, // Город получателя
					'CustomerCountryCode' => 'RUS', // Код страны
					'CustomerEmail' => $oShopOrder->email, // Email получателя
					'CustomerFirstName' => $oShopOrder->name, // Имя получателя
					'CustomerMobilePhoneNumber' => $oShopOrder->phone, // Номер мобильного телефона получателя
					'CustomerSurname' => $oShopOrder->surname, // Фамилия получателя
					'InsuranceCurrency' => 'RUB', // Валюта страховой стоимости посылки
					'InsuranceValue' => 0, // Страховая стоимость посылки
					'ParcelshopCode' => $this->_getPropertyValue($oShopOrder, 'point_number'), // Номер ПВЗ
					'Services' => array('DIRECT_DELIVERY') // Список услуг
				)
			)
		);

		$oCreatePreadviceResponse = $this->_makeRequest('CreatePreadvice', $data);

		$oCreatePreadviceResult = $oCreatePreadviceResponse->CreatePreadviceResult;
		$oErrorCode = $oCreatePreadviceResult->xpath('a:PreadviceResult/a:ErrorCode');

		if ($oCreatePreadviceResponse && count($oErrorCode) && strval($oErrorCode[0]) == 'Success')
		{
			$this->_log('Заказ #' . $oShopOrder->invoice . ' отправлен в Hermes.');
		}
		else
		{
			$this->_reportError($oCreatePreadviceResult->asXml());
		}
	}

	/**
	 * Выполняет запрос к API;
	 *
	 * @param  string  $method
	 * @param  array  $data=array()
	 * @return SimpleXmlElement|NULL
	 */
	protected function _makeRequest($method, $data = array())
	{
		$authToken = base64_encode($this->_apiLogin . ':' . $this->_apiPassword);

		$oCoreHttp = Core_Http::instance()
			->clear()
			->method('POST')
			->url($this->_apiUrl . $method)
			->contentType('application/json')
			->additionalHeader('Authorization', 'Basic ' . $authToken)
			->rawData(json_encode($data))
			->timeout($this->_apiTimeout)
			->execute();

		try
		{
			$oResponse = new \SimpleXmlElement($oCoreHttp->getBody(), LIBXML_NOWARNING | LIBXML_NOERROR);

			return $oResponse;
		}
		catch (\Exception $e) {
			$this->_reportError($oCoreHttp->getBody());
		}

		return NULL;
	}
}