<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Реализует отгрузку товара в ТК PickPoint.
 */
class Shop_Order_Dispatcher_Pickpoint extends Shop_Order_Dispatcher
{
	// Вид отправления
	const POSTAGE_TYPE_PREPAY = 10001;
	const POSTAGE_TYPE_NOT_PREPAY = 10003;

	// Тип сдачи отправления
	const GETTING_TYPE_CORIER = 101;
	const GETTING_TYPE_SERVICE_CENTER = 102;

	// Тип оплаты отправления
	const PAY_TYPE_DEFAULT = 1;

	/**
	 * URL для доступа к API.
	 *
	 * Тестовый: http://e-solution.pickpoint.ru/apitest/
	 * Продакшн: http://e-solution.pickpoint.ru/api/
	 *
	 * @var string
	 */
	protected $_apiUrl = 'http://e-solution.pickpoint.ru/api/';

	/**
	 * Логин для доступа к API.
	 *
	 * Для теста:
	 * apitest
	 *
	 * @var string
	 */
	protected $_apiLogin;

	/**
	 * Пароль для доступа к API.
	 *
	 * Для теста:
	 * apitest
	 *
	 * @var string
	 */
	protected $_apiPassword;

	/**
	 * ИКН.
	 *
	 * @var string
	 */
	protected $_apiIkn;

	/**
	 * {@inherit}
	 */
	public function __construct(array $aConfig)
	{
		$this->_apiUrl = Core_Array::get($aConfig, 'url');
		$this->_apiLogin = Core_Array::get($aConfig, 'login');
		$this->_apiPassword = Core_Array::get($aConfig, 'password');
		$this->_apiIkn = Core_Array::get($aConfig, 'ikn');
	}

	/**
	 * {@inherit}
	 */
	protected function _dispatch(Shop_Order_Model $oShopOrder)
	{
		$postamatNumber = self::_getPropertyValue($oShopOrder, 'point_number');

		if (!$postamatNumber)
		{
			return;
		}

		// Логинимся в API
		$oLoginResponse = $this->_makeRequest('login', array(
			'Login' => $this->_apiLogin,
			'Password' => $this->_apiPassword
		));

		if (isset($oLoginResponse->SessionId) && $oLoginResponse->SessionId)
		{
			$aRecipientName = array($oShopOrder->name, $oShopOrder->surname, $oShopOrder->patronymic);
			$aRecipientName = array_filter($aRecipientName);

			$aSendingData = array(
				'Sendings' => array(
					array(
						'EDTN' => $oShopOrder->invoice,
						'IKN' => $this->_apiIkn,
						'Invoice' => array(
							'Description' => $oShopOrder->description, // Описание
							'PostamatNumber' => $postamatNumber, // Номер постамата
							'RecipientName' => implode(' ', $aRecipientName),
							'MobilePhone' => $oShopOrder->phone, // Номер телефона получателя
							'PostageType' => self::POSTAGE_TYPE_PREPAY, // Тип услуги
							'GettingType' => self::GETTING_TYPE_CORIER, // Тип сдачи отправления
							'PayType' => self::PAY_TYPE_DEFAULT, // Тип оплаты
							'Sum' => 0, // Сумма к оплате
							'Width' => $this->_aParams['width'], // Ширина
							'Height' => $this->_aParams['height'], // Высота
							'Depth' => $this->_aParams['depth'], // Глубина
							'SenderCode' => $oShopOrder->invoice, // Номер заказа магазина
							'Email' => $oShopOrder->email // Электронная почта
						)
					)
				)
			);

			$oCreateSendingResponse = $this->_makeRequest('createsending', $aSendingData + array(
				'SessionId' => $oLoginResponse->SessionId
			));

			if (!empty($oCreateSendingResponse->CreatedSendings))
			{
				$this->_log('Заказ #' . $oShopOrder->invoice . ' отправлен в PickPoint.');
			}
			else
			{
				$this->_reportError(json_encode($oCreateSendingResponse));
			}
		}
		else
		{
			$this->_reportError(json_encode($oLoginResponse));
		}
	}

	/**
	 * Выполняет запрос к API.
	 *
	 * @param  string  $resource
	 * @param  array  $data
	 * @return StdClass|NULL
	 */
	protected function _makeRequest($resource, $data)
	{
		$Core_Http = Core_Http::instance()
			->clear()
			->method('POST')
			->url($this->_apiUrl . $resource)
			->contentType('application/json')
			->additionalHeader('Accept', 'application/json')
			->rawData(json_encode($data))
			->timeout($this->_apiTimeout)
			->execute();

		return json_decode($Core_Http->getBody());
	}
}